local M = {}
--[[
    Hello, this is the source code for the More Features Mod, you may look and modify stuff as you wish, 
    but you are not allowed to re-publish the mod or any code that you used in this without my permission.

    StinchinStein
]]--
local drivetrain = require("drivetrain")
local electrics = require("electrics")
--Sound variables

local loadedSounds = {}
M.engineNode = nil

local function createSFXSource(filename, description, SFXProfileName, nodeID)
  local snd = gameEngine:createSFXSource(filename, description, SFXProfileName, true)
  obj:attachSFXNode(snd, M.engineNode)
  table.insert(loadedSounds, snd)
  return snd
end

local function playPitchSound(soundName, nodeID, volume, pitch) 
    obj:playSFXOnce(soundName, M.engineNode, volume, pitch)
end

local function destroySfx()
    for _,s in pairs(loadedSounds) do
        gameEngine:deleteSFXSource(s, true)
    end
    loadedSounds = {}
end

local function createSfx()
    local cameraNode = 0
    if v.data.camerasInternal ~= nil then
        local k, v = next(v.data.camerasInternal)
        if v ~= nil then
            cameraNode = v.camNodeID
        end
    end
    M.engineNode = cameraNode
    if v.data.engine ~= nil and v.data.engine.torqueReactionNodes_nodes then
        local t = v.data.engine.torqueReactionNodes_nodes
        if #t > 0 and v.data.nodes[t[1]] ~= nil then
            M.engineNode = t[1]
        end
    end

    createSFXSource('art/sound/new_sounds/headlight_switch.ogg', 'AudioDefault3D', "HeadlightSwitchSound", 1)
    createSFXSource('art/sound/new_sounds/blinker_auto_off.ogg', 'AudioDefault3D', "BlinkerAutoOffSound", 1)
    createSFXSource('art/sound/new_sounds/gavril_blinker_off.ogg', 'AudioDefault3D', "GavrilBlinkerOffSound", 1)
    createSFXSource('art/sound/new_sounds/gavril_blinker_on.ogg', 'AudioDefault3D', "GavrilBlinkerOnSound", 1)

    createSFXSource('art/sound/new_sounds/horn_new.ogg', 'AudioDefault3D', "HornNewSound", 1)

    createSFXSource('art/sound/new_sounds/SHIFTER_INTO_GEAR_1.ogg', 'AudioDefault3D', "ShiftGear1", 1)
    createSFXSource('art/sound/new_sounds/SHIFTER_INTO_GEAR_2.ogg', 'AudioDefault3D', "ShiftGear2", 1)
    createSFXSource('art/sound/new_sounds/SHIFTER_TO_N.ogg', 'AudioDefault3D', "ShiftGearNeutral", 1)
    createSFXSource('art/sound/new_sounds/starter_dead.ogg', 'AudioDefault3D', "StarterDeadSound", 1)

    createSFXSource('art/sound/new_sounds/car_ignition.ogg', 'AudioDefault3D', "CarIgnitionSound", 1)
    createSFXSource('art/sound/new_sounds/gear_whine1.ogg', 'AudioDefault3D', "GearWhineSound1", 1)
    createSFXSource('art/sound/new_sounds/brake_squeal.ogg', 'AudioDefault3D', "BrakeSquealSound", 1)
    createSFXSource('art/sound/new_sounds/parking_brake_engage.ogg', 'AudioDefault3D', "ParkingBrakeEngage", 1)

    createSFXSource('art/sound/new_sounds/radiator_fan.ogg', 'AudioDefault3D', "RadiatorFanSound", 1)
	--createSFXSource('art/sound/new_sounds/radiator_fan_broken.ogg', 'AudioDefault3D', "RadiatorFanBrokenSound", 1)

    createSFXSource('art/sound/new_sounds/ENGINE_2_CRANKING_LOOP.ogg', 'AudioDefault3D', "StartEngineSound", 1)
    createSFXSource('art/sound/new_sounds/ENGINE_3_START.ogg', 'AudioDefault3D', "EngineStartedSound", 1)
    createSFXSource('art/sound/new_sounds/ENGINE_STOP.ogg', 'AudioDefault3D', "EngineStopSound", 1)
    createSFXSource('art/sound/new_sounds/gear_grind.ogg', 'AudioDefault3D', "GearGrindSound", 1)

    createSFXSource('art/sound/new_sounds/horn_alert.ogg', 'AudioDefault3D', "HornAlertSound", 1)
    createSFXSource('art/sound/new_sounds/rc_beep.ogg', 'AudioDefault3D', "RCBeepSound", 1)
    createSFXSource('art/sound/new_sounds/rc_battery_critical.ogg', 'AudioDefault3D', "RCBatteryCriticalSound", 1)

    createSFXSource('art/sound/new_sounds/blinker_click.ogg', 'AudioDefault3D', "BlinkerClickSound", 1)
    createSFXSource('art/sound/new_sounds/blinker_on.ogg', 'AudioDefault3D', "BlinkerOnSound", 1)
    createSFXSource('art/sound/new_sounds/blinker_off.ogg', 'AudioDefault3D', "BlinkerOffSound", 1)

    createSFXSource('art/sound/new_sounds/key_insert.ogg', 'AudioDefault3D', "KeyInsertSound", 1)

    createSFXSource('art/sound/new_sounds/beep_reverse.ogg', 'AudioDefault3D', "ReverseBeepSound", 1)
    createSFXSource('art/sound/new_sounds/horn.ogg', 'AudioDefault3D', "HornSound", 1)
    createSFXSource('art/sound/new_sounds/police_siren.ogg', 'AudioDefault3D', "PoliceSirenSound", 1)

    createSFXSource('art/sound/new_sounds/airbrake_release.ogg', 'AudioDefault3D', "AirbrakeReleaseSound", 1)
end

local function init()
    destroySfx()
	createSfx()
end

local function callShift(toGear)
    local eng = v.data.engine
    local r = math.random(1, 2)

    if(MoreFeaturesMod.configurationFile.shiftSounds and MoreFeaturesMod.globalConfiguration.shiftSounds) then
        if(toGear == 0) then
            playPitchSound("ShiftGearNeutral", 1, 1, 1) --speed / 70
        elseif(toGear <= eng.fwdGearCount and toGear >= (v.data.engine.transmissionType == "automatic" and -3 or -eng.revGearCount)) then

            if(r == 1) then
                playPitchSound("ShiftGear1", 1, 1, 1) --speed / 70
            else 
                playPitchSound("ShiftGear2", 1, 1, 1) --speed / 70
            end
        end
    end
end

local function update(dt)

end




M.playPitchSound = playPitchSound
M.callShift = callShift
M.init = init
M.update = update

return M