local M = {}
--[[
    Hello, this is the source code for the More Features Mod, you may look and modify stuff as you wish, 
    but you are not allowed to re-publish the mod or any code that you used in this without my permission.

    StinchinStein
]]--
local drivetrain = require("drivetrain")
local electrics = require("electrics")
local SoundEffects = require("SoundEffects")


local allModes = {0,1,2,3}
local autoModes = {2,3}
local manualModes = {2,1,0}
local activeModes = allModes

local function getShifterModeName(mode)
    if mode == 0 then return "Manual" end
    if mode == 1 then return "Manual (clutch assist)" end
    if mode == 2 then return "Arcade" end
    if mode == 3 then return "Automatic" end
    return "Unknown shifting mode"
end
local function getShifterModeIndex(mode)
    for i,m in ipairs(activeModes) do
        if m == mode then return i end
    end
    return nil
end

local function smartShifterModeHShifter(gear)
    local newModeIndex = nil
    if gear == 0 then return end
    if gear > 3 then
        if not newModeIndex then newModeIndex = getShifterModeIndex(0) end -- manual full
        if not newModeIndex then newModeIndex = getShifterModeIndex(1) end -- manual autoclutch
        if not newModeIndex then gui.message("Cannot shift to gear #"..gear.." with an automatic gearbox", 5, "shiftermode") end
    else
        if not newModeIndex then newModeIndex = getShifterModeIndex(3) end -- real auto
        if not newModeIndex then newModeIndex = getShifterModeIndex(0) end -- manual full
        if not newModeIndex then newModeIndex = getShifterModeIndex(1) end -- manual autoclutch
    end
    if not newModeIndex then return end
    drivetrain.setShifterMode(newModeIndex)
    gui.message("H-shifter has been used, so shifting mode has been set to: '"..getShifterModeName(activeModes[newModeIndex]).."'", 10, "shiftermode")
end
local function smartShifterModePaddles()
    local newModeIndex = nil
    if not newModeIndex then newModeIndex = getShifterModeIndex(3) end -- real auto
    if not newModeIndex then newModeIndex = getShifterModeIndex(1) end -- manual autoclutch
    if not newModeIndex then newModeIndex = getShifterModeIndex(0) end -- manual full
    if not newModeIndex then return end
    drivetrain.setShifterMode(newModeIndex)
    gui.message("Shifter paddle/button has been used, so shifting mode has been set to: '"..getShifterModeName(activeModes[newModeIndex]).."'", 10, "shiftermode")
end

local function shiftUpReset()
    if drivetrain.shifterMode == 2 then smartShifterModePaddles() end -- from arcade auto to a suitable mode
    drivetrain.shifterPosition = drivetrain.shifterPosition + 1
    SoundEffects.callShift(drivetrain.shifterPosition)
end

local function shiftDownReset()
    local eng = v.data.engine
    if drivetrain.shifterMode == 2 then smartShifterModePaddles() end -- from arcade auto to a suitable mode
    drivetrain.shifterPosition = drivetrain.shifterPosition - 1
    SoundEffects.callShift(drivetrain.shifterPosition)
end
local function shiftToGearReset(val)
    local eng = v.data.engine
    if drivetrain.shifterMode == 2 then smartShifterModePaddles() end -- from arcade auto to a suitable mode
    drivetrain.shifterPosition = val
    SoundEffects.callShift(drivetrain.shifterPosition)
end

local function shiftUp()
    if drivetrain.shifterMode == 2 then smartShifterModePaddles() end -- from arcade auto to a suitable mode
    local speed = obj:getVelocity():length() * 2.236 -- speed
    local eng = v.data.engine
    if(M.ableToShift or M.targetShiftPosition + 1 == 0) then
        M.targetShiftPosition = M.targetShiftPosition + 1
        SoundEffects.callShift(M.targetShiftPosition)
    else
        M.transmissionWear = M.transmissionWear + 5
        if(speed > 40) then pitchSpeed = 40 else pitchSpeed = speed end
        drivetrain.engineAV = drivetrain.engineAV - v.data.engine.maxAV / 6
        SoundEffects.playPitchSound("GearGrindSound", 1, 0.3 + speed / 70, 1 + pitchSpeed / 650) --speed / 70
    end
    M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
    --gui.message("Target Shift: ".. M.targetShiftPosition)
end

local function shiftDown()
    if drivetrain.shifterMode == 2 then smartShifterModePaddles() end -- from arcade auto to a suitable mode
    local speed = obj:getVelocity():length() * 2.236 -- speed
    local eng = v.data.engine
    if(M.ableToShift or M.targetShiftPosition - 1 == 0) then
        M.targetShiftPosition = M.targetShiftPosition - 1
        SoundEffects.callShift(M.targetShiftPosition)
    else
        M.transmissionWear = M.transmissionWear + 5
        if(speed > 40) then pitchSpeed = 40 else pitchSpeed = speed end
        drivetrain.engineAV = drivetrain.engineAV - v.data.engine.maxAV / 6
        SoundEffects.playPitchSound("GearGrindSound", 1, 0.3 + speed / 70, 1 + pitchSpeed / 650) --speed / 70
    end
    M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
    --gui.message("Target Shift: ".. M.targetShiftPosition)
end

local function shiftToGear(val)
    local speed = obj:getVelocity():length() * 2.236 -- speed
    if(M.ableToShift or val == 0) then
        M.targetShiftPosition = val
        SoundEffects.callShift(M.targetShiftPosition)
    else
        M.transmissionWear = M.transmissionWear + 5
        if(speed > 40) then pitchSpeed = 40 else pitchSpeed = speed end
        drivetrain.engineAV = drivetrain.engineAV - v.data.engine.maxAV / 6
        SoundEffects.playPitchSound("GearGrindSound", 1, 0.3 + speed / 70, 1 + pitchSpeed / 650) --speed / 70
    end
    M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
    --gui.message("Target Shift: ".. M.targetShiftPosition)
end

--=================================================================================================================================

local function init()
	M.ableToShift = false
	M.targetShiftPosition = 0
	M.transmissionWear = 0
    M.transmissionDamagedMsg = false

    if v.data.engine.transmissionType == "automatic" then
        activeModes = autoModes
    end

    if v.data.engine.transmissionType == "manual" then
        activeModes = manualModes
    end

end

local function disable()
    drivetrain.shiftUp = shiftUpReset
    drivetrain.shiftDown = shiftDownReset
    drivetrain.shiftToGear = shiftToGearReset
end
local function update(dt)
	local eng = v.data.engine
	M.ableToShift = false

    if(MoreFeaturesMod.EngineSystem.engineIsOff) then
        M.ableToShift = true
    end

    M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )

    drivetrain.shiftUp = shiftUp
    drivetrain.shiftDown = shiftDown
    drivetrain.shiftToGear = shiftToGear
    drivetrain.shifterPosition = M.targetShiftPosition
    
    if((input.clutch > 0.4 or MoreFeaturesMod.engineIsOff) and M.transmissionWear < 100) then
        M.ableToShift = true
    elseif(M.transmissionWear >= 100 and not M.transmissionDamagedMsg) then
        gui.message("Transmission Damaged, You Can No Longer Shift Gears.", 5, "disp.gearDmg")
        M.transmissionDamagedMsg = true
    end
end


M.init = init
M.disable = disable
M.update = update

return M